var sidebarInstall = {

  rdfS    : null,
  panelURN: null,
  ncURI   : null,
  listURN : null,
  
  panelDS : null,
  panelCT : null,
  panelRes: null,
  
  initDS : function () {
    const PANELS_RDF_FILE        = "UPnls"; /* directory services property to find panels.rdf */
    const DIR_SERV_CONTRACTID    = "@mozilla.org/file/directory_service;1";
    const IO_SERV_CONTRACTID     = "@mozilla.org/network/io-service;1";
    const RDF_CONTRACTID         = "@mozilla.org/rdf/rdf-service;1";
    const CONTAINER_CONTRACTID   = "@mozilla.org/rdf/container;1";

    const nsIProperties          = Components.interfaces.nsIProperties;
    const nsIFile                = Components.interfaces.nsIFile;
    const nsIIOService           = Components.interfaces.nsIIOService;
    const nsIFileProtocolHandler = Components.interfaces.nsIFileProtocolHandler;
    const nsIRDFService          = Components.interfaces.nsIRDFService;
    const nsIRDFResource         = Components.interfaces.nsIRDFResource;
    const nsIRDFContainer        = Components.interfaces.nsIRDFContainer;

    const kNCURI                 = "http://home.netscape.com/NC-rdf#";
    const kSidebarURNPanelList   = "urn:sidebar:current-panel-list";
    const kSidebarURN3rdParty    = "urn:sidebar:3rdparty-panel";

    this.rdfS = Components.classes[RDF_CONTRACTID].getService(nsIRDFService);
    this.panelURN = kSidebarURN3rdParty;
    this.ncURI = kNCURI;
    this.listURN = kSidebarURNPanelList;

    var dataS = this.panelDS;
    if (!dataS) {
      var dirS = Components.classes[DIR_SERV_CONTRACTID]
                 .getService(nsIProperties);
      var dsFile = dirS.get(PANELS_RDF_FILE, nsIFile);
      if (!dsFile.exists()) {
        return null;
      }
      var ioS = Components.classes[IO_SERV_CONTRACTID]
                .getService(nsIIOService);
      var fileH = ioS.getProtocolHandler("file")
                  .QueryInterface(nsIFileProtocolHandler);
      var dsUri = fileH.getURLSpecFromFile(dsFile);
      dataS = this.rdfS.GetDataSourceBlocking(dsUri);
    }
    this.panelDS = dataS;
    var panelList = dataS.GetTarget(this.rdfS.GetResource(kSidebarURNPanelList),
                                    this.rdfS.GetResource(kNCURI + "panel-list"),
                                    true);
    panelList.QueryInterface(nsIRDFResource);
    var container = Components.classes[CONTAINER_CONTRACTID]
                    .createInstance(nsIRDFContainer);
    container.Init(dataS, panelList);
    this.panelCT = container;

    return dataS;
  },
  
  isSidebarInstalled: function(sidebarURL) {
    if (!this.panelDS) {
      this.initDS();
    }
    this.panelRes = this.rdfS.GetResource(this.panelURN + ":" + sidebarURL);
    return this.panelCT.IndexOf(this.panelRes) != -1;
  },

  installSidebar: function(sidebarTitle, sidebarURL) {
    if (this.isSidebarInstalled(sidebarURL))
      return false;
    this.panelDS.Assert(this.panelRes,
                        this.rdfS.GetResource(this.ncURI + "title"),
                        this.rdfS.GetLiteral(sidebarTitle),
                        true);
    this.panelDS.Assert(this.panelRes,
                        this.rdfS.GetResource(this.ncURI + "content"),
                        this.rdfS.GetLiteral(sidebarURL),
                        true);
    this.panelCT.AppendElement(this.panelRes);

    this.forceSidebarRefresh();

    return true;
  },

  forceSidebarRefresh : function () {
    if (!this.panelDS)
      this.initDS;
    var listRes = this.rdfS.GetResource(this.listURN);
    var refreshRes = this.rdfS.GetResource(this.ncURI + "refresh");
    var trueRes = this.rdfS.GetLiteral("true");
    this.panelDS.Assert(listRes, refreshRes, trueRes, true);
    this.panelDS.Unassert(listRes, refreshRes, trueRes);
  }

}